<?php

/**
 * @file
 * Administration file for the Format Currencies project.
 */

/**
 * General settings form.
 */
function format_currency_settings(&$form_state) {
  $currencies = array();
  $query = db_query('SELECT code, name FROM {currencies} WHERE status = '. FORMAT_CURRENCY_ACTIVE);
  while ($currency = db_fetch_object($query)) {
    $currencies[$currency->code] = $currency->name;
  }

  if (!empty($currencies)) {
    $form['format_currency_test_wrapper'] = array(
      '#type' => 'fieldset',
      '#title' => t('Test output:'),
      '#prefix' => '<div id="format-currency-wrapper" class="container-inline">',
      '#suffix' => '</div>',
    );
    $form['format_currency_test_wrapper']['format_currency_test_submit'] = array(
      '#type' => 'submit',
      '#ahah' => array(
        'path' => 'admin/settings/format_currency/ahah',
        'wrapper' => 'format-currency-test',
      ),
      '#value' => t('Display'),
    );
    $form['format_currency_test_wrapper']['format_currency_test_select'] = array(
      '#type' => 'select',
      '#title' => t('Currency'),
      '#options' => $currencies,
    );
    $form['format_currency_test_wrapper']['format_currency_test'] = array(
      '#type' => 'textfield',
      '#description' => theme('format_currency', 100, 'GBP'),
      '#prefix' => '<div id="format-currency-test">',
      '#suffix' => '</div>',
      '#default_value' => isset($form_state['post']['format_currency_test']) ? $form_state['post']['format_currency_test'] : 100,
      '#size' => 10,
    );
  }
  $form['format_currency_display_code'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display Currency Code'),
    '#description' => t('Whether to display the Currency Code (eg. "USD &#36;123.45") before a formatted price.'),
    '#default_value' => variable_get('format_currency_display_code', FALSE),
  );
  $form['format_currency_display_symbol'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use symbol rather than HTML entity'),
    '#description' => t('The database table holds both a symbol and HTML entity for Currencies, check the box to use the symbol over the entity.'),
    '#default_value' => variable_get('format_currency_display_symbol', FALSE),
  );
  $form['format_currency_round_whole_numbers'] = array(
    '#type' => 'checkbox',
    '#title' => t('Round whole numbers'),
    '#description' => t('Instead of showing &pound;100.00, checking this box will display the price as &pound;100. NOTE this only affects whole numbers &pound;100.01 will still display as normal.'),
    '#default_value' => variable_get('format_currency_round_whole_numbers', FALSE),
  );
  $form['format_currency_prefer_coinage'] = array(
    '#type' => 'checkbox',
    '#title' => t('Prefer coinage when displaying a price less than ONE unit'),
    '#description' => t('If displaying a price which is less than ONE unit, checking this box will use the coinage symbol if it is present in the configuration.
      <br />Eg. Rather than display &#36;0.99, display 99&cent;.'),
    '#default_value' => variable_get('format_currency_prefer_coinage', FALSE),
  );

  return system_settings_form($form);
}

/**
 * Overview of currencies currently in the database.
 */
function format_currency_admin(&$form_state) {
  $form['currencies']['#tree'] = TRUE;

  $fields = drupal_schema_fields_sql('currencies');
  $query = pager_query('SELECT '. implode(', ', $fields) .' FROM {currencies}'. tablesort_sql(_format_currency_admin_header()), 25);

  while ($currency = db_fetch_object($query)) {
    foreach ($fields as $field) {
      $form['currencies'][$currency->code][$field] = array(
        '#type' => 'markup',
        '#value' => $currency->$field,
      );
    }
  }

  return $form;
}

/**
 * Confirmation form to check if a user wants to rebuild their currencies table.
 */
function format_currency_repopulate(&$form_state) {
  return confirm_form(array(), t('Are you sure you want rebuild your currency table?'), 'admin/settings/format_currency/overview', t('<strong>Caution</strong>: This action is permanent and cannot be undone, it will erase all customisation you have made to your currency configuration.'));
}

/**
 * Submission callback for the format_currency_repopulate form.
 */
function format_currency_repopulate_submit($form, &$form_state) {
  $batch = array(
    'title' => t('Rebuilding currency table'),
    'operations' => array(
      array('_format_currency_batch_repopulate', array(25)),
    ),
  );

  // Start up the batch process.
  batch_set($batch);
  batch_process('admin/settings/format_currency/overview');
}

/**
 * Adds a new currency to the database.
 */
function format_currency_add(&$form_state) {
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency name'),
    '#maxlength' => 64,
  );
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['num'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency number'),
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 32,
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => FORMAT_CURRENCY_LEFT,
  );
  $form['coinage_symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage Symbol'),
    '#size' => 15,
    '#maxlength' => 15,
  );
  $form['coinage_entity'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
  );
  $form['coinage_position'] = array(
    '#type' => 'select',
    '#title' => t('Coinage Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => FORMAT_CURRENCY_RIGHT,
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => ',',
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => '.',
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => 2,
  );
  $form['base'] = array(
    '#type' => 'textfield',
    '#title' => t('Base'),
    '#required' => TRUE,
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => 100,
  );
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('Currency Status'),
    '#options' => array(
      FORMAT_CURRENCY_EXPIRED => t('Expired'),
      FORMAT_CURRENCY_ACTIVE => t('Active'),
    ),
    '#default_value' => FORMAT_CURRENCY_ACTIVE,
  );

  return confirm_form($form, t('Add New Currency'), 'admin/settings/format_currency/overview', ' ', t('Add'));
}

/**
 * Validation callback for the format_currency_add form.
 */
function format_currency_add_validate(&$form, &$form_state) {
  if (db_result(db_query("SELECT 1 FROM {currencies} WHERE code = '%s'", $form_state['values']['code']))) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!empty($form_state['values']['num']) && !is_numeric($form_state['values']['num'])) {
    form_set_error('decimal_places', t('Currency number field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['base'])) {
    form_set_error('base', t('Base field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_add form.
 */
function format_currency_add_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values']);
  drupal_set_message(t('Successfully added currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * Form to edit a given currency in the database.
 */
function format_currency_edit(&$form_state, $currency) {
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency name'),
    '#maxlength' => 64,
    '#default_value' => $currency->name,
  );
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->code,
  );
  $form['num'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency number'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->num,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->symbol,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 32,
    '#default_value' => $currency->entity,
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => $currency->symbol_position,
  );
  $form['coinage_symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage Symbol'),
    '#size' => 15,
    '#maxlength' => 15,
    '#default_value' => $currency->coinage_symbol,
  );
  $form['coinage_entity'] = array(
    '#type' => 'textfield',
    '#title' => t('Coinage HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
    '#default_value' => $currency->coinage_entity,
  );
  $form['coinage_position'] = array(
    '#type' => 'select',
    '#title' => t('Coinage Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => $currency->coinage_position,
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->thousands_separator,
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->decimal_separator,
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => $currency->decimal_places,
  );
  $form['base'] = array(
    '#type' => 'textfield',
    '#title' => t('Base'),
    '#required' => TRUE,
    '#size' => 6,
    '#maxlength' => 6,
    '#default_value' => $currency->base,
  );
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('Currency Status'),
    '#options' => array(
      FORMAT_CURRENCY_EXPIRED => t('Expired'),
      FORMAT_CURRENCY_ACTIVE => t('Active'),
    ),
    '#default_value' => $currency->status,
  );
  $form['old_code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Edit Currency %currency:', array('%currency' => $currency->code)), 'admin/settings/format_currency/overview', ' ', t('Edit'));
}

/**
 * Validation callback for the format_currency_edit form.
 */
function format_currency_edit_validate(&$form, &$form_state) {
  if ($form_state['values']['old_code'] != $form_state['values']['code'] && db_result(db_query("SELECT 1 FROM {currencies} WHERE code = '%s'", $form_state['values']['code']))) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!empty($form_state['values']['num']) && !is_numeric($form_state['values']['num'])) {
    form_set_error('decimal_places', t('Currency number field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['base'])) {
    form_set_error('base', t('Base field can be numeric only.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_edit form.
 */
function format_currency_edit_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values'], 'code');
  drupal_set_message(t('Successfully altered currency %currency.', array('%currency' => $form_state['values']['old_code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * Form to confirm deletion of a currency from the database.
 */
function format_currency_delete(&$form_state, $currency) {
  $form['code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Are you sure you want to delete currency %currency?', array('%currency' => $currency->code)), 'admin/settings/format_currency/overview');
}

/**
 * Submission callback for the format_currency_delete form.
 */
function format_currency_delete_submit($form, &$form_state) {
  db_query("DELETE FROM {currencies} WHERE code = '%s'", $form_state['values']['code']);
  drupal_set_message(t('Successfully deleted currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * AHAH page callback to alter the format_currency_test description to show themed price output.
 */
function format_currency_ahah() {
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);

  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  $form['#programmed'] = $form['#redirect'] = FALSE;

  drupal_process_form($form_id, $form, $form_state);
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);

  $display = $form['format_currency_test_wrapper']['format_currency_test'];
  $display['#description'] = theme('format_currency', $form_state['post']['format_currency_test'], $form_state['post']['format_currency_test_select']);

  // Remove the prefix/suffix wrapper so we don't double it up.
  unset($display['#prefix'], $display['#suffix']);

  // Clear any unwanted messages.
  drupal_get_messages();

  $output = drupal_render($display);

  // Final rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
}

/**
 * Themes the admin overview form.
 */
function theme_format_currency_admin($form) {
  drupal_add_css(drupal_get_path('module', 'format_currency') .'/format_currency.css');

  $output = '';
  $rows = array();
  $header = _format_currency_admin_header();

  foreach (element_children($form['currencies']) as $code) {
    $row = array();
    $code = drupal_render($form['currencies'][$code]['code']);
    $status = (boolean) drupal_render($form['currencies'][$code]['status']);
    $num = (int) drupal_render($form['currencies'][$code]['num']);
    $symbol_position = (boolean) drupal_render($form['currencies'][$code]['symbol_position']);
    $coinage_position = (boolean) drupal_render($form['currencies'][$code]['coinage_position']);
    if ($links = drupal_render($form['currencies'][$code]['links'])) {
      _format_currency_create_links($links);
    }

    $row[] = '<a href="#'. check_plain($code) .'" name="'. check_plain($code) .'" class="format-currency-tooltip">'. check_plain($code) . ($status == FORMAT_CURRENCY_EXPIRED ? '*' : '') .'<span>'. check_plain(drupal_render($form['currencies'][$code]['name'])) .'</span></a>';
    $row[] = !empty($num) ? str_pad($num, 3, 0, STR_PAD_LEFT) : '';
    $row[] = check_plain(drupal_render($form['currencies'][$code]['symbol']));
    $row[] = check_plain(drupal_render($form['currencies'][$code]['entity']));
    $row[] = _format_currency_symbol_position($symbol_position);
    $row[] = check_plain(drupal_render($form['currencies'][$code]['coinage_symbol']));
    $row[] = check_plain(drupal_render($form['currencies'][$code]['coinage_entity']));
    $row[] = _format_currency_symbol_position($coinage_position);
    $row[] = check_plain(drupal_render($form['currencies'][$code]['thousands_separator']));
    $row[] = check_plain(drupal_render($form['currencies'][$code]['decimal_separator']));
    $row[] = check_plain(drupal_render($form['currencies'][$code]['decimal_places']));
    $row[] = check_plain(drupal_render($form['currencies'][$code]['base']));
    $row[] = is_array($links) ? theme('item_list', $links) : '';
    $row[] = l(t('edit'), "admin/settings/format_currency/$code/edit", array('query' => drupal_get_destination()));
    $row[] = l(t('delete'), "admin/settings/format_currency/$code/delete", array('query' => drupal_get_destination()));

    $rows[] = $row;
  }

  if (empty($rows)) {
    $rows[][] = array('data' => t('No currencies are currently in the database.'), 'colspan' => count($header));
  }

  $output .= theme('table', $header, $rows, array('id' => 'format-currency-admin-table'));
  $output .= '<div id="format-currency-admin-links">';
  $output .= l(t('Add new currency'), 'admin/settings/format_currency/add');
  $output .= '</div>';
  $output .= theme('pager', array(), 25);

  // Render any remaining form elements.
  $output .= drupal_render($form);

  return $output;
}

/**
 * Helper function for converting the links into something useful.
 */
function _format_currency_create_links(&$links) {
  $links = unserialize($links);
  foreach ($links as &$link) {
    preg_match("/^(?:http:\/\/)?(?:www\.)?([^\/]+)/", $link, $match);
    switch ($match[1]) {
      case 'xe.com':
        $link = l(t('XE'), $link, array('external' => TRUE));
        break;
      case 'gocurrency.com':
        $link = l(t('GoCurrency'), $link, array('external' => TRUE));
        break;
      case 'en.wikipedia.org':
        $link = l(t('Wikipedia'), $link, array('external' => TRUE));
        break;
      default:
        $link = l($link, $link, array('external' => TRUE));
        break;
    }
  }
}

/**
 * Helper form containing the admin overview table header.
 */
function _format_currency_admin_header() {
  return array(
    array('data' => t('Code'), 'field' => 'code', 'sort' => 'asc'),
    array('data' => t('Num'), 'field' => 'num'),
    t('Symbol'),
    t('Entity'),
    t('Symbol Position'),
    t('Coinage Symbol'),
    t('Coinage Entity'),
    t('Coinage Position'),
    t('Thousands Separator'),
    t('Decimal Separator'),
    t('Decimal Places'),
    t('Base'),
    t('Links'),
    array('data' => t('Operations'), 'colspan' => 2),
  );
}